import logging
import cgi
import os
from urllib.parse import urljoin, urlparse, unquote
from importlib import metadata

import requests

from .authentication import CZDSAuthentication


class CZDSClient(CZDSAuthentication):
    BASE_URL = 'https://czds-api.icann.org'
    ZONE_DOWNLOAD_LINKS_LIST_URL = urljoin(BASE_URL, '/czds/downloads/links')

    def __init__(self, username, password):
        super().__init__(username=username, password=password)

    def _do_request(self, method, url, stream=False):
        if not self._is_authenticated():
            logging.debug('Not authenticated. Attempting authentication.')
            self._authenticate()

        headers = {
            'Authorization': 'Bearer {}'.format(self._token),
            'User-Agent': '{} / {}'.format(
                metadata.distribution('pyczds').name, metadata.distribution('pyczds').version
            )
        }

        request = requests.Request(method.upper(), url=url, headers=headers)
        prepared_request = request.prepare()
        self._preprocess_request(prepared_request)

        response = requests.Session().send(prepared_request, stream=stream)
        self._preprocess_response(response, stream)

        return response

    def get_zone_download_links(self):
        logging.debug('About to request zonefile links list.')

        link_list = self._do_request('get', self.ZONE_DOWNLOAD_LINKS_LIST_URL).json()

        if len(link_list) == 0:
            raise Exception('This account does not seem to have access to any zonefiles.')

        return link_list

    def head_zonefile(self, zonefile_link):
        logging.debug('About to request headers for zonefile {}.'.format(zonefile_link.split('/')[-1]))

        headers = self._do_request('head', zonefile_link).headers

        return headers

    def get_zonefile(self, zonefile_link, download_dir=''):
        if not download_dir or (download_dir and len(download_dir) == 0):
            download_dir = os.getcwd()

        url_parsed = urlparse(zonefile_link)
        filename = unquote(os.path.basename(url_parsed.path))

        logging.debug('About to start download for zonefile {} to directory {}.'.format(
            filename, download_dir)
        )

        with self._do_request('get', zonefile_link, stream=True) as response:
            value, params = cgi.parse_header(response.headers['Content-Disposition'])
            if value.lower() != 'attachment':
                raise ValueError(
                    'Received unexpected value in Content-Disposition header: {}.'.format(
                        response.headers['Content-Disposition'])
                )
            if 'filename' not in params.keys():
                raise ValueError(
                    'No filename passed in Content-Disposition header: {}.'.format(
                        response.headers['Content-Disposition'])
                )

            filename = params['filename']
            filepath = os.path.join(download_dir, filename)

            logging.debug('Streaming file to {}.'.format(filepath))

            with open(filepath, 'wb') as f:
                for chunk in response.iter_content(chunk_size=8192):
                    f.write(chunk)

            logging.debug('Completed download of zonefile {}.'.format(filepath))



import logging
import re

import requests


class CZDSHelpers(object):

    def _is_email_address(self, address_string):
        pat = r'\b[A-Za-z0-9._%+-]+@[A-Za-z0-9.-]+\.[A-Z|a-z]{2,}\b'
        return re.match(pat, address_string)

    def _preprocess_request(self, request):
        logging.debug(
            'About to send {} request to link {} with headers {}.'.format(request.method, request.url, request.headers)
        )

    def _preprocess_response(self, response, stream=False):
        logging.debug('Returned status code {}.'.format(response.status_code))
        logging.debug('Received headers {}.'.format(response.headers))

        if not stream:
            logging.debug('Received raw response {}.'.format(response.text))

        if response.status_code != 200:
            logging.error('Error during request to API. Returned status code {}.'.format(response.status_code))
            if response.status_code == 400:
                raise requests.HTTPError('Request is malformed.')
            if response.status_code == 401:
                raise requests.HTTPError('Invalid credentials provided, or no valid bearer token provided.')
            if response.status_code == 403:
                raise requests.HTTPError(
                    'Requested zone does not exist, or authenticated user does not have access to the zone.'
                )
            if response.status_code == 409:
                raise requests.HTTPError(
                    'The authenticated user has not accepted the new T&Cs. Please log in to the portal, and accept the T&Cs.'
                )
            if response.status_code == 415:
                raise requests.HTTPError('Unsupported content type header sent.')
            if response.status_code == 429:
                raise requests.HTTPError('Too many authentication attempts from the same IP address.')
            if response.status_code == 500:
                raise requests.HTTPError('Internal service error.')
